export declare class Try<T> {
    private value;
    private executionStack;
    private internalError?;
    private constructor();
    static combine<T extends any[], R>(...args: [...{
        [K in keyof T]: Try<T[K]>;
    }, (...values: T) => R]): Try<R>;
    static success<T>(value: T): Try<T>;
    static failure<T>(error: Error): Try<T>;
    static of<T>(fn: () => T | Promise<T>): Try<Awaited<T>>;
    private setValue;
    private setError;
    private executeElement;
    private runExecutionStack;
    run(): Promise<Try<T>>;
    get(): Promise<T>;
    getOrElse<U>(defaultValue: U): Promise<U | T>;
    getOrElseGet<U>(fn: (ex: Error) => U): Promise<T | U>;
    getOrElseThrow<U>(fn: (error: Error) => U): Promise<T | U>;
    map<U>(fn: (value: T) => U): Try<Awaited<U>>;
    mapIf<U>(predicateFunc: (value: T) => boolean | Promise<boolean>, fn: (value: T) => U): Try<Awaited<U>>;
    flatMap<U>(fn: (value: T) => Try<U> | Promise<Try<U>>): Try<Awaited<U>>;
    flatMapIf<U>(predicateFunc: (value: T) => boolean | Promise<boolean>, fn: (value: T) => Try<U> | Promise<Try<U>>): Try<Awaited<U>>;
    mapFailure<E extends Error, U extends Error>(func: (ex: E) => U | Promise<U>): Try<T>;
    mapFailureWith<E extends Error, U extends Error>(errorType: new (...args: any[]) => E, func: (ex: E) => U | Promise<U>): Try<T>;
    filter(predicateFunc: (value: T) => boolean | Promise<boolean>, throwbackFunction?: (value: T) => Error): Try<T>;
    filterNot(predicateFunc: (value: T) => boolean | Promise<boolean>, throwbackFunction?: (value: T) => Error): Try<T>;
    peek(fn: (value: T) => void): Try<T>;
    andThen(fn: (value: T) => any): Try<T>;
    andFinally(fn: () => any): Try<T>;
    recover<U>(fn: (error: Error) => U): Try<T | U>;
    recoverWith<U>(fn: (error: Error) => Try<U>): Try<U | T>;
    isSuccess(): boolean;
    isFailure(): boolean;
    onSuccess(fn: (value: T) => void): Try<T>;
    onFailure(fn: (ex: Error) => void): Try<T>;
    getCause(): Error | undefined;
}
