"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Try = void 0;
const NoSuchElementException_1 = require("../exceptions/NoSuchElementException");
const TryFunctions = {
    OF: "OF",
    COMBINE: 'COMBINE',
    MAP: 'MAP',
    MAPIF: 'MAPIF',
    ANDTHEN: 'ANDTHEN',
    ANDFINALLY: 'ANDFINALLY',
    FLATMAP: 'FLATMAP',
    FLATMAPIF: 'FLATMAPIF',
    FILTER: 'FILTER',
    FILTERNOT: 'FILTERNOT',
    PEEK: 'PEEK',
    RECOVER: 'RECOVER',
    RECOVERWITH: 'RECOVERWITH',
    ONSUCCESS: 'ONSUCCESS',
    ONFAILURE: 'ONFAILURE',
    MAPFAILURE: 'MAPFAILURE',
    MAPFAILUREWITH: 'MAPFAILUREWITH',
};
class Try {
    constructor(initExecution) {
        this.executionStack = [];
        if (initExecution)
            this.executionStack.push(initExecution);
    }
    static combine(...args) {
        const trys = args.slice(0, -1);
        const func = args[args.length - 1];
        return new Try({
            name: TryFunctions.COMBINE,
            functionData: { func: () => func(...trys.map(tryObj => tryObj.value)), trys },
            returning: true
        });
    }
    //Static methods to create a Try object
    static success(value) {
        return new Try().setValue(value);
    }
    static failure(error) {
        return new Try().setError(error);
    }
    static of(fn) {
        return new Try({
            name: TryFunctions.OF,
            functionData: { func: fn },
            returning: true
        });
    }
    //Private methods to modify the internal state
    setValue(value) {
        this.value = value;
        return this;
    }
    setError(err) {
        this.internalError = err;
        return this;
    }
    executeElement(executionElement) {
        return __awaiter(this, void 0, void 0, function* () {
            if (executionElement.returning) {
                return this.value = executionElement.name === TryFunctions.OF
                    ? yield executionElement.functionData.func()
                    : yield executionElement.functionData.func(this.value);
            }
            yield executionElement.functionData.func(this.value);
        });
    }
    runExecutionStack() {
        return __awaiter(this, void 0, void 0, function* () {
            for (const executionElement of this.executionStack) {
                try {
                    if (executionElement.name === TryFunctions.MAP) {
                        if (this.isSuccess())
                            yield this.executeElement(executionElement);
                    }
                    else if (executionElement.name === TryFunctions.MAPIF) {
                        if (this.isSuccess()) {
                            if (yield executionElement.functionData.fallbackFunction(this.value)) {
                                yield this.executeElement(executionElement);
                            }
                        }
                    }
                    else if (executionElement.name === TryFunctions.FLATMAP) {
                        if (this.isSuccess()) {
                            const tryObject = yield executionElement.functionData.func(this.value);
                            this.value = yield tryObject.get();
                        }
                    }
                    else if (executionElement.name === TryFunctions.FLATMAPIF) {
                        if (this.isSuccess()) {
                            if (yield executionElement.functionData.fallbackFunction(this.value)) {
                                const tryObject = yield executionElement.functionData.func(this.value);
                                this.value = yield tryObject.get();
                            }
                        }
                    }
                    else if (executionElement.name === TryFunctions.ANDTHEN) {
                        if (this.isSuccess())
                            yield this.executeElement(executionElement);
                    }
                    else if (executionElement.name === TryFunctions.ANDFINALLY) {
                        yield this.executeElement(executionElement);
                    }
                    else if (executionElement.name === TryFunctions.FILTER) {
                        if (this.isSuccess()) {
                            if (yield executionElement.functionData.func(this.value)) {
                                throw yield executionElement.functionData.fallbackFunction(this.value);
                            }
                        }
                    }
                    else if (executionElement.name === TryFunctions.FILTERNOT) {
                        if (this.isSuccess()) {
                            if (!(yield executionElement.functionData.func(this.value))) {
                                throw yield executionElement.functionData.fallbackFunction(this.value);
                            }
                        }
                    }
                    else if (executionElement.name === TryFunctions.PEEK) {
                        if (this.isSuccess())
                            yield executionElement.functionData.func(this.value);
                    }
                    else if (executionElement.name === TryFunctions.RECOVER) {
                        if (this.isFailure()) {
                            this.value = yield executionElement.functionData.func(this.internalError);
                            this.internalError = undefined;
                        }
                    }
                    else if (executionElement.name === TryFunctions.RECOVERWITH) {
                        if (this.isFailure()) {
                            const tryObject = yield executionElement.functionData.func(this.internalError);
                            this.internalError = undefined;
                            this.value = yield tryObject.get();
                        }
                    }
                    else if (executionElement.name === TryFunctions.ONSUCCESS) {
                        if (this.isSuccess())
                            yield executionElement.functionData.func(this.value);
                    }
                    else if (executionElement.name === TryFunctions.ONFAILURE) {
                        if (this.isFailure())
                            yield executionElement.functionData.func(this.internalError);
                    }
                    else if (executionElement.name === TryFunctions.COMBINE) {
                        const values = yield Promise.all(executionElement.functionData.trys.map((tryObject) => __awaiter(this, void 0, void 0, function* () { return yield tryObject.run(); })));
                        for (const v of values) {
                            if (v.isFailure()) {
                                this.internalError = v.internalError;
                                break;
                            }
                        }
                        this.value = yield executionElement.functionData.func(...values);
                    }
                    else if (executionElement.name === TryFunctions.MAPFAILURE) {
                        if (this.isFailure()) {
                            this.internalError = yield executionElement.functionData.func(this.internalError);
                        }
                    }
                    else if (executionElement.name === TryFunctions.MAPFAILUREWITH) {
                        if (this.isFailure() && this.internalError instanceof executionElement.functionData.errorType) {
                            this.internalError = yield executionElement.functionData.func(this.internalError);
                        }
                    }
                    else {
                        yield this.executeElement(executionElement);
                    }
                }
                catch (ex) {
                    this.internalError = ex;
                }
            }
        });
    }
    //----- Public interface -----
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runExecutionStack();
            return this;
        });
    }
    get() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runExecutionStack();
            if (this.isFailure())
                throw this.internalError;
            return this.value;
        });
    }
    getOrElse(defaultValue) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runExecutionStack();
            return this.isFailure() ? defaultValue : this.value;
        });
    }
    getOrElseGet(fn) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runExecutionStack();
            return this.isFailure() ? yield fn(this.internalError) : this.value;
        });
    }
    getOrElseThrow(fn) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.runExecutionStack();
            if (this.isFailure())
                throw fn(this.internalError);
            return this.value;
        });
    }
    map(fn) {
        this.executionStack.push({
            name: TryFunctions.MAP,
            functionData: { func: fn },
            returning: true
        });
        return this;
    }
    mapIf(predicateFunc, fn) {
        this.executionStack.push({
            name: TryFunctions.MAPIF,
            functionData: { func: fn, fallbackFunction: predicateFunc },
            returning: true
        });
        return this;
    }
    flatMap(fn) {
        this.executionStack.push({
            name: TryFunctions.FLATMAP,
            functionData: { func: fn },
            returning: true
        });
        return this;
    }
    flatMapIf(predicateFunc, fn) {
        this.executionStack.push({
            name: TryFunctions.FLATMAPIF,
            functionData: { func: fn, fallbackFunction: predicateFunc },
            returning: true
        });
        return this;
    }
    mapFailure(func) {
        this.executionStack.push({
            name: TryFunctions.MAPFAILURE,
            functionData: { func: func },
            returning: false
        });
        return this;
    }
    mapFailureWith(errorType, func) {
        this.executionStack.push({
            name: TryFunctions.MAPFAILUREWITH,
            functionData: { func: func, errorType: errorType },
            returning: false
        });
        return this;
    }
    filter(predicateFunc, throwbackFunction) {
        this.executionStack.push({
            name: TryFunctions.FILTER,
            functionData: { func: predicateFunc, fallbackFunction: throwbackFunction !== null && throwbackFunction !== void 0 ? throwbackFunction : ((value) => { throw new NoSuchElementException_1.NoSuchElementException(`Predicate does not hold for ${value}`); }) },
            returning: false
        });
        return this;
    }
    filterNot(predicateFunc, throwbackFunction) {
        this.executionStack.push({
            name: TryFunctions.FILTERNOT,
            functionData: { func: predicateFunc, fallbackFunction: throwbackFunction !== null && throwbackFunction !== void 0 ? throwbackFunction : ((value) => { throw new NoSuchElementException_1.NoSuchElementException(`Predicate does not hold for ${value}`); }) },
            returning: false
        });
        return this;
    }
    peek(fn) {
        this.executionStack.push({
            name: TryFunctions.PEEK,
            functionData: { func: fn },
            returning: false
        });
        return this;
    }
    andThen(fn) {
        this.executionStack.push({
            name: TryFunctions.ANDTHEN,
            functionData: { func: fn },
            returning: false
        });
        return this;
    }
    andFinally(fn) {
        this.executionStack.push({
            name: TryFunctions.ANDFINALLY,
            functionData: { func: fn },
            returning: false
        });
        return this;
    }
    recover(fn) {
        this.executionStack.push({
            name: TryFunctions.RECOVER,
            functionData: { func: fn },
            returning: true
        });
        return this;
    }
    recoverWith(fn) {
        this.executionStack.push({
            name: TryFunctions.RECOVERWITH,
            functionData: { func: fn },
            returning: true
        });
        return this;
    }
    isSuccess() {
        return this.internalError === undefined;
    }
    isFailure() {
        return this.internalError !== undefined;
    }
    onSuccess(fn) {
        this.executionStack.push({
            name: TryFunctions.ONSUCCESS,
            functionData: { func: fn },
            returning: false
        });
        return this;
    }
    onFailure(fn) {
        this.executionStack.push({
            name: TryFunctions.ONFAILURE,
            functionData: { func: fn },
            returning: false
        });
        return this;
    }
    getCause() {
        return this.internalError;
    }
}
exports.Try = Try;
